@@ -6,6 +6,7 @@ import Paths_shake
 import Development.Shake.Types
 import Development.Shake.Core
 import Development.Shake.Demo
+import Development.Shake.FilePath
 import Development.Shake.Rules.File
 import Development.Shake.Progress
 import Development.Shake.Shake
@@ -86,12 +87,12 @@ shakeArgs opts rules = shakeArgsWith opts [] f
 --   As an example of a build system that can use either @gcc@ or @distcc@ for compiling:
 --
 -- @
---import System.Console.GetOpt
+-- import System.Console.GetOpt
 --
---data Flags = DistCC deriving Eq
---flags = [Option \"\" [\"distcc\"] (NoArg $ Right DistCC) \"Run distributed.\"]
+-- data Flags = DistCC deriving Eq
+-- flags = [Option \"\" [\"distcc\"] (NoArg $ Right DistCC) \"Run distributed.\"]
 --
---main = 'shakeArgsWith' 'shakeOptions' flags $ \\flags targets -> return $ Just $ do
+-- main = 'shakeArgsWith' 'shakeOptions' flags $ \\flags targets -> return $ Just $ do
 --     if null targets then 'want' [\"result.exe\"] else 'want' targets
 --     let compiler = if DistCC \`elem\` flags then \"distcc\" else \"gcc\"
 --     \"*.o\" '%>' \\out -> do
@@ -115,7 +116,12 @@ shakeArgsWith baseOpts userOptions rules = do
         progressRecords = [x | ProgressRecord x <- flagsExtra]
         changeDirectory = listToMaybe [x | ChangeDirectory x <- flagsExtra]
         printDirectory = last $ False : [x | PrintDirectory x <- flagsExtra]
-        shakeOpts = foldl' (flip ($)) baseOpts flagsShake
+        oshakeOpts = foldl' (flip ($)) baseOpts flagsShake
+        shakeOpts = oshakeOpts {shakeLintInside = map (toStandard . normalise . addTrailingPathSeparator) $
+                                                  shakeLintInside oshakeOpts
+                               ,shakeLintIgnore = map toStandard $
+                                                  shakeLintIgnore oshakeOpts
+                               }
 
     -- error if you pass some clean and some dirty with specific flags
     errs <- return $ errs ++ flagsError ++ ["cannot mix " ++ a ++ " and " ++ b | a:b:_ <-
@@ -126,7 +132,7 @@ shakeArgsWith baseOpts userOptions rules = do
         showHelp
         exitFailure
 
-    if Help `elem` flagsExtra then do
+    if Help `elem` flagsExtra then
         showHelp
      else if Version `elem` flagsExtra then
         putStrLn $ "Shake build system, version " ++ showVersion version
@@ -152,14 +158,14 @@ shakeArgsWith baseOpts userOptions rules = do
                 Just d -> bracket_ (getDataFileName "html" >> setCurrentDirectory d) (setCurrentDirectory curdir)
         shakeOpts <- if null progressRecords then return shakeOpts else do
             t <- offsetTime
-            return shakeOpts{shakeProgress = \p -> do
+            return shakeOpts{shakeProgress = \p ->
                 bracket
                     (forkIO $ shakeProgress shakeOpts p)
                     killThread
                     $ const $ progressDisplay 1 (const $ return ()) $ do
                         p <- p
                         t <- t
-                        forM_ progressRecords $ \file -> do
+                        forM_ progressRecords $ \file ->
                             appendFile file $ show (t,p) ++ "\n"
                         return p
             }
@@ -180,7 +186,7 @@ shakeArgsWith baseOpts userOptions rules = do
             in case res of
                 Left err ->
                     if Exception `elem` flagsExtra then
-                        throw err
+                        throwIO err
                     else do
                         putStrLn $ esc "31" $ show err
                         exitFailure
@@ -197,7 +203,7 @@ shakeArgsWith baseOpts userOptions rules = do
             putStr $ unlines $ ("Usage: " ++ progName ++ " [options] [target] ...") : "Options:" : showOptDescr opts
 
         wrap :: (a -> b) -> OptDescr (Either String a) -> OptDescr (Either String b)
-        wrap f = fmapOptDescr (either Left (Right . f))
+        wrap = fmapOptDescr . fmap
 
 
 showOptDescr :: [OptDescr a] -> [String]
@@ -227,7 +233,7 @@ fmapOptDescr f (Option a b c d) = Option a b (g c) d
 --   in 'ShakeOptions'. The command line flags are @make@ compatible where possbile, but additional
 --   flags have been added for the extra options Shake supports.
 shakeOptDescrs :: [OptDescr (Either String (ShakeOptions -> ShakeOptions))]
-shakeOptDescrs = [fmapOptDescr (either Left (Right . snd)) o | (True, o) <- shakeOptsEx]
+shakeOptDescrs = [fmapOptDescr (fmap snd) o | (True, o) <- shakeOptsEx]
 
 data Extra = ChangeDirectory FilePath
            | Version
@@ -270,6 +276,7 @@ shakeOptsEx =
     ,yes $ Option ""  ["digest-and"] (NoArg $ Right ([], \s -> s{shakeChange=ChangeModtimeAndDigest})) "Files change when modtime and digest change."
     ,yes $ Option ""  ["digest-and-input"] (NoArg $ Right ([], \s -> s{shakeChange=ChangeModtimeAndDigestInput})) "Files change on modtime (and digest for inputs)."
     ,yes $ Option ""  ["digest-or"] (NoArg $ Right ([], \s -> s{shakeChange=ChangeModtimeOrDigest})) "Files change when modtime or digest change."
+    ,yes $ Option ""  ["digest-not"] (NoArg $ Right ([], \s -> s{shakeChange=ChangeModtime})) "Files change when modtime changes."
     ,no  $ Option ""  ["exception"] (NoArg $ Right ([Exception], id)) "Throw exceptions directly."
     ,yes $ Option ""  ["flush"] (intArg 1 "flush" "N" (\i s -> s{shakeFlush=Just i})) "Flush metadata every N seconds."
     ,yes $ Option ""  ["never-flush"] (noArg $ \s -> s{shakeFlush=Nothing}) "Never explicitly flush metadata."
@@ -277,7 +284,7 @@ shakeOptsEx =
     ,yes $ Option "j" ["jobs"] (optIntArg 0 "jobs" "N" $ \i s -> s{shakeThreads=fromMaybe 0 i}) "Allow N jobs/threads at once [default CPUs]."
     ,yes $ Option "k" ["keep-going"] (noArg $ \s -> s{shakeStaunch=True}) "Keep going when some targets can't be made."
     ,yes $ Option "l" ["lint"] (noArg $ \s -> s{shakeLint=Just LintBasic}) "Perform limited validation after the run."
-    ,yes $ Option ""  ["lint-tracker"] (noArg $ \s -> s{shakeLint=Just LintTracker}) "Use tracker.exe to do validation."
+    ,yes $ Option ""  ["lint-fsatrace"] (noArg $ \s -> s{shakeLint=Just LintFSATrace}) "Use fsatrace to do validation."
     ,yes $ Option ""  ["no-lint"] (noArg $ \s -> s{shakeLint=Nothing}) "Turn off --lint."
     ,yes $ Option ""  ["live"] (OptArg (\x -> Right ([], \s -> s{shakeLiveFiles=shakeLiveFiles s ++ [fromMaybe "live.txt" x]})) "FILE") "List the files that are live [to live.txt]."
     ,yes $ Option "m" ["metadata"] (reqArg "PREFIX" $ \x s -> s{shakeFiles=x}) "Prefix for storing metadata files."
@@ -334,7 +341,7 @@ shakeOptsEx =
         outputDebug output Nothing = output
         outputDebug output (Just file) = \v msg -> do
             when (v /= Diagnostic) $ output v msg
-            appendFile file $ unescape msg
+            appendFile file $ unescape msg ++ "\n"
 
         outputColor output v msg = output v $ escape "34" msg
 
