@@ -124,11 +124,9 @@ copyFile' old new = do
 copyFileChanged :: FilePath -> FilePath -> Action ()
 copyFileChanged old new = do
     need [old]
-    eq <- liftIO $ doesFileExist new &&^ do
-        withBinaryFile old ReadMode $ \h1 -> withBinaryFile new ReadMode $ \h2 ->
-            liftM2 (==) (hFileSize h1) (hFileSize h2) &&^
-                liftM2 (==) (BS.hGetContents h1) (BS.hGetContents h2)
-    unless eq $ do
+    -- in newer versions of the directory package we can use copyFileWithMetadata which (we think) updates
+    -- the timestamp as well and thus no need to read the source file twice.
+    unlessM (liftIO $ doesFileExist new &&^ fileEq old new) $ do
         putLoud $ "Copying from " ++ old ++ " to " ++ new
         -- copyFile does a lot of clever stuff with permissions etc, so make sure we just reuse it
         liftIO $ copyFile old new
@@ -139,7 +137,7 @@ readFile' :: FilePath -> Action String
 readFile' x = need [x] >> liftIO (readFile x)
 
 -- | Write a file, lifted to the 'Action' monad.
-writeFile' :: FilePath -> String -> Action ()
+writeFile' :: MonadIO m => FilePath -> String -> m ()
 writeFile' name x = liftIO $ writeFile name x
 
 
@@ -149,12 +147,12 @@ readFileLines :: FilePath -> Action [String]
 readFileLines = fmap lines . readFile'
 
 -- | A version of 'writeFile'' which writes out a list of lines.
-writeFileLines :: FilePath -> [String] -> Action ()
+writeFileLines :: MonadIO m => FilePath -> [String] -> m ()
 writeFileLines name = writeFile' name . unlines
 
 
 -- | Write a file, but only if the contents would change.
-writeFileChanged :: FilePath -> String -> Action ()
+writeFileChanged :: MonadIO m => FilePath -> String -> m ()
 writeFileChanged name x = liftIO $ do
     b <- doesFileExist name
     if not b then writeFile name x else do
