@@ -32,7 +32,7 @@ type Capture a = (a -> IO ()) -> IO ()
 
 -- See https://ghc.haskell.org/trac/ghc/ticket/11555
 catchSafe :: IO a -> (SomeException -> IO a) -> IO a
-catchSafe a b = join (evaluate a) `catch_` b
+catchSafe = catch_
 
 -- | Run and then call a continuation.
 runRAW :: ro -> rw -> RAW ro rw a -> Capture (Either SomeException a)
