@@ -1,5 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable, RecordWildCards, CPP, ForeignFunctionInterface #-}
-{-# LANGUAGE ViewPatterns #-}
+{-# LANGUAGE DeriveDataTypeable, RecordWildCards, CPP, ViewPatterns, ForeignFunctionInterface #-}
 
 -- | Progress tracking
 module Development.Shake.Progress(
@@ -37,9 +36,13 @@ import Prelude
 import Foreign
 import Foreign.C.Types
 
-type LPCSTR = Ptr CChar
+#ifdef x86_64_HOST_ARCH
+#define CALLCONV ccall
+#else
+#define CALLCONV stdcall
+#endif
 
-foreign import stdcall "Windows.h SetConsoleTitleA" c_setConsoleTitle :: LPCSTR -> IO Bool
+foreign import CALLCONV "Windows.h SetConsoleTitleA" c_setConsoleTitle :: Ptr CChar -> IO Bool
 
 #endif
 
@@ -47,9 +50,10 @@ foreign import stdcall "Windows.h SetConsoleTitleA" c_setConsoleTitle :: LPCSTR
 ---------------------------------------------------------------------
 -- PROGRESS TYPES - exposed to the user
 
--- | Information about the current state of the build, obtained by passing a callback function
---   to 'Development.Shake.shakeProgress'. Typically a program will use 'progressDisplay' to poll this value and produce
---   status messages, which is implemented using this data type.
+-- | Information about the current state of the build, obtained by either passing a callback function
+--   to 'Development.Shake.shakeProgress' (asynchronous output) or 'Development.Shake.getProgress'
+--   (synchronous output). Typically a build system will pass 'progressDisplay' to 'Development.Shake.shakeProgress',
+--   which will poll this value and produce status messages.
 data Progress = Progress
     {isFailure :: !(Maybe String) -- ^ Starts out 'Nothing', becomes 'Just' a target name if a rule fails.
     ,countSkipped :: {-# UNPACK #-} !Int -- ^ Number of rules which were required, but were already in a valid state.
@@ -274,7 +278,7 @@ generateHTML :: [(FilePath, [ProgressEntry])] -> IO LBS.ByteString
 generateHTML xs = do
     htmlDir <- getDataFileName "html"
     report <- LBS.readFile $ htmlDir </> "progress.html"
-    let f name | name == "progress-data.js" = return $ LBS.pack $ "var shake =\n" ++ generateJSON xs
+    let f name | name == "progress-data.js" = return $ LBS.pack $ "var progress =\n" ++ generateJSON xs
                | name == "version.js" = return $ LBS.pack $ "var version = " ++ show (showVersion version)
                | otherwise = LBS.readFile $ htmlDir </> name
     runTemplate f report
@@ -298,9 +302,9 @@ jsonObject xs = "{" ++ intercalate ", " [show a ++ ":" ++ b | (a,b) <- xs] ++ "}
 
 {-# NOINLINE xterm #-}
 xterm :: Bool
-xterm = System.IO.Unsafe.unsafePerformIO $
+xterm = unsafePerformIO $
     -- Terminal.app uses "xterm-256color" as its env variable
-    catch_ (fmap ("xterm" `isPrefixOf`) $ getEnv "TERM") $
+    catch_ (("xterm" `isPrefixOf`) <$> getEnv "TERM") $
     \e -> return False
 
 
@@ -344,7 +348,9 @@ progressProgram = do
                            in if null b then "" else reverse $ takeWhile isDigit $ reverse a
                 let key = (failure, perc)
                 same <- atomicModifyIORef ref $ \old -> (Just key, old == Just key)
-                let state = if perc == "" then "NoProgress" else if failure then "Error" else "Normal"
+                let state | perc == "" = "NoProgress"
+                          | failure = "Error"
+                          | otherwise = "Normal"
                 rawSystem exe $ ["--title=" ++ msg, "--state=" ++ state] ++ ["--value=" ++ perc | perc /= ""]
                 return ()
 
