@@ -1,4 +1,3 @@
-{-# LANGUAGE MultiParamTypeClasses, GeneralizedNewtypeDeriving, DeriveDataTypeable #-}
 
 module Development.Shake.Rules.OrderOnly(
      orderOnly, orderOnlyBS
@@ -22,11 +21,10 @@ import qualified Data.ByteString.Char8 as BS
 -- @
 --
 --   If @header.h@ is included by @source.c@ then the call to 'needMakefileDependencies' will cause
---   it to be added as a real dependency. If it isn't, then the rule won't rebuild if it changes,
---   and you will have lost some opportunity for parallelism.
+--   it to be added as a real dependency. If it isn't, then the rule won't rebuild if it changes.
 orderOnly :: [FilePath] -> Action ()
-orderOnly = unsafeIgnoreDependencies . need
+orderOnly = orderOnlyAction . need
 
 
 orderOnlyBS :: [BS.ByteString] -> Action ()
-orderOnlyBS = unsafeIgnoreDependencies . needBS
+orderOnlyBS = orderOnlyAction . needBS
