@@ -1,4 +1,4 @@
-{-# LANGUAGE ScopedTypeVariables, PatternGuards, NamedFieldPuns, FlexibleInstances, MultiParamTypeClasses #-}
+{-# LANGUAGE ScopedTypeVariables, PatternGuards, RecordWildCards, FlexibleInstances, MultiParamTypeClasses #-}
 {-
 This module stores the meta-data so its very important its always accurate
 We can't rely on getting any exceptions or termination at the end, so we'd better write out a journal
@@ -12,6 +12,7 @@ module Development.Shake.Storage(
 import General.Binary
 import Development.Shake.Types
 import General.Timing
+import General.FileLock
 
 import Data.Tuple.Extra
 import Control.Exception.Extra
@@ -61,7 +62,10 @@ withStorage
     -> w                        -- ^ Witness
     -> (Map k v -> (k -> v -> IO ()) -> IO a)  -- ^ Execute
     -> IO a
-withStorage ShakeOptions{shakeVerbosity,shakeOutput,shakeVersion,shakeVersionIgnore,shakeFlush,shakeFiles,shakeStorageLog} diagnostic witness act = do
+withStorage ShakeOptions{..} diagnostic witness act = do
+  diagnostic $ "Before fileLock on " ++ shakeFiles </> ".shake.lock"
+  withLockFile (shakeFiles </> ".shake.lock") $ do
+    diagnostic "After fileLock"
     let dbfile = shakeFiles </> ".shake.database"
         bupfile = shakeFiles </> ".shake.backup"
     createDirectoryIfMissing True shakeFiles
@@ -70,7 +74,7 @@ withStorage ShakeOptions{shakeVerbosity,shakeOutput,shakeVersion,shakeVersionIgn
     b <- doesFileExist bupfile
     when b $ do
         unexpected "Backup file exists, restoring over the previous file\n"
-        diagnostic $ "Backup file move to original"
+        diagnostic "Backup file move to original"
         ignore $ removeFile dbfile
         renameFile bupfile dbfile
 
@@ -94,7 +98,7 @@ withStorage ShakeOptions{shakeVerbosity,shakeOutput,shakeVersion,shakeVersionIgn
             continue h Map.empty
          else
             -- make sure you are not handling exceptions from inside
-            join $ handleJust (\e -> if asyncException e then Nothing else Just e) (\err -> do
+            join $ handleBool (not . asyncException) (\err -> do
                 msg <- showException err
                 outputErr $ unlines $
                     ("Error when reading Shake database " ++ dbfile) :
@@ -147,7 +151,7 @@ withStorage ShakeOptions{shakeVerbosity,shakeOutput,shakeVersion,shakeVersionIgn
                                 hSetFileSize h $ now - slop
                                 hSeek h AbsoluteSeek $ now - slop
                                 hFlush h
-                                diagnostic $ "Drop complete"
+                                diagnostic "Drop complete"
                             return $ continue h mp
                          else do
                             addTiming "Database compression"
