@@ -1,10 +1,10 @@
-{-# LANGUAGE BangPatterns, CPP #-}
 
 module General.Extra(
     getProcessorCount,
     randomElem,
     showQuote,
     withs,
+    maximum', maximumBy'
     ) where
 
 import Control.Exception.Extra
@@ -14,10 +14,8 @@ import System.Environment.Extra
 import System.IO.Extra
 import System.IO.Unsafe
 import System.Random
-#if __GLASGOW_HASKELL__ >= 704
 import Control.Concurrent
-import Foreign.C.Types
-#endif
+import GHC.Conc
 
 
 ---------------------------------------------------------------------
@@ -31,23 +29,15 @@ showQuote xs | any isSpace xs = "\"" ++ concatMap (\x -> if x == '\"' then "\"\"
 ---------------------------------------------------------------------
 -- System.Info
 
-#if __GLASGOW_HASKELL__ >= 704
--- Use the underlying GHC function
-foreign import ccall getNumberOfProcessors :: IO CInt
-#endif
-
-
 {-# NOINLINE getProcessorCount #-}
 getProcessorCount :: IO Int
 -- unsafePefromIO so we cache the result and only compute it once
 getProcessorCount = let res = unsafePerformIO act in return res
     where
         act =
-#if __GLASGOW_HASKELL__ >= 704
             if rtsSupportsBoundThreads then
-                fmap fromIntegral $ getNumberOfProcessors
+                fmap fromIntegral $ getNumProcessors
             else
-#endif
                 handle_ (const $ return 1) $ do
                     env <- lookupEnv "NUMBER_OF_PROCESSORS"
                     case env of
@@ -72,3 +62,11 @@ randomElem xs = do
 withs :: [(a -> r) -> r] -> ([a] -> r) -> r
 withs [] act = act []
 withs (f:fs) act = f $ \a -> withs fs $ \as -> act $ a:as
+
+
+-- See https://ghc.haskell.org/trac/ghc/ticket/10830 - they broke maximumBy
+maximumBy' :: (a -> a -> Ordering) -> [a] -> a
+maximumBy' cmp = foldl1' $ \x y -> if cmp x y == GT then x else y
+
+maximum' :: Ord a => [a] -> a
+maximum' = maximumBy' compare
