@@ -21,10 +21,11 @@ import qualified Data.ByteString.Lazy.Char8 as LBS
 import Prelude
 
 
-main = shaken test $ \args obj -> do
-    let helper = [toNative $ obj "shake_helper" <.> exe]
+main = shakenCwd test $ \args obj -> do
+    -- shake_helper must be in a subdirectory so we can test placing that subdir on the $PATH
+    let helper = toNative $ obj "helper/shake_helper" <.> exe
     let name !> test = do want [name | null args || name `elem` args]
-                          name ~> do need [obj "shake_helper" <.> exe]; test
+                          name ~> do need [obj "helper/shake_helper" <.> exe]; test
 
     let helper_source = unlines
             ["import Control.Concurrent"
@@ -50,8 +51,12 @@ main = shaken test $ \args obj -> do
             ,"        hFlush stderr"
             ]
 
-    obj "shake_helper.hs" %> \out -> do need ["src/Test/Command.hs"]; writeFileChanged out helper_source
-    obj "shake_helper" <.> exe %> \out -> do need [obj "shake_helper.hs"]; cmd (Cwd $ obj "") "ghc --make" "shake_helper.hs -o shake_helper"
+    obj "shake_helper.hs" %> \out -> do
+        need ["../../src/Test/Command.hs"]
+        writeFileChanged out helper_source
+    [obj "helper/shake_helper" <.> exe, obj "shake_helper.o", obj "shake_helper.hi"] &%> \_ -> do
+        need [obj "shake_helper.hs"]
+        cmd (Cwd $ obj "") "ghc --make" "shake_helper.hs -o helper/shake_helper"
 
     "capture" !> do
         (Stderr err, Stdout out) <- cmd helper ["ostuff goes here","eother stuff here"]
@@ -69,11 +74,13 @@ main = shaken test $ \args obj -> do
 
     "cwd" !> do
         -- FIXME: Linux searches the Cwd argument for the file, Windows searches getCurrentDirectory
-        helper <- liftIO $ canonicalizePath $ obj "shake_helper" <.> exe
+        helper <- liftIO $ canonicalizePath $ obj "helper/shake_helper" <.> exe
         Stdout out <- cmd (Cwd $ obj "") helper "c"
-        liftIO $ (===) (trim out) =<< canonicalizePath (dropTrailingPathSeparator $ obj "")
+        let norm = fmap dropTrailingPathSeparator . canonicalizePath . trim
+        liftIO $ join $ liftM2 (===) (norm out) (norm $ obj "")
 
     "timeout" !> do
+        opts <- getShakeOptions
         offset <- liftIO offsetTime
         Exit exit <- cmd (Timeout 2) helper "w20"
         t <- liftIO offset
@@ -88,10 +95,22 @@ main = shaken test $ \args obj -> do
         Stdout out <- cmd (AddEnv "FOO" "GOODBYE SHAKE") Shell helper "vFOO"
         liftIO $ out === "GOODBYE SHAKE\n"
 
+    "space" !> do
+        Stdout out <- cmd helper ["oSPACE 1"]
+        liftIO $ out === "SPACE 1\n"
+        Stdout out <- cmd Shell helper "\"oSPACE 2\""
+        liftIO $ out === "SPACE 2\n"
+        whenM (liftIO hasTracker) $ do
+            Stdout out <- cmd Shell AutoDeps helper "\"oSPACE 2\""
+            liftIO $ out === "SPACE 2\n"
+        (Stdout (), CmdLine x) <- cmd helper ["oSPACE 3","oDIRECT"]
+        unless (" \"oSPACE 3\" oDIRECT" `isSuffixOf` replace "\'" "\"" x) $
+            fail $ "Invalid CmdLine, " ++ x
+
     "path" !> do
-        let path = AddPath [dropTrailingPathSeparator $ obj ""] []
-        unit $ cmd $ obj "shake_helper"
-        unit $ cmd $ obj "shake_helper" <.> exe
+        let path = AddPath [dropTrailingPathSeparator $ obj "helper"] []
+        unit $ cmd $ obj "helper/shake_helper"
+        unit $ cmd $ obj "helper/shake_helper" <.> exe
         unit $ cmd path Shell "shake_helper"
         unit $ cmd path "shake_helper"
 
@@ -122,8 +141,10 @@ main = shaken test $ \args obj -> do
             [s ++ " = " ++ showDuration d | (s,d) <- [("String",t1),("ByteString",t2),("File",t3)]]
 
     "stdin" !> do
-        Stdout (x :: String) <- cmd helper "i" (Stdin "hello ") (StdinBS $ LBS.pack "world")
-        liftIO $ x === "hello world"
+        withTempFile $ \file -> do
+            liftIO $ writeFile file " "
+            Stdout (x :: String) <- cmd helper "i" (Stdin "hello") (FileStdin file) (StdinBS $ LBS.pack "world")
+            liftIO $ x === "hello world"
 
     "async" !> do
         let file = obj "async.txt"
@@ -135,6 +156,9 @@ main = shaken test $ \args obj -> do
 
 test build obj = do
     -- reduce the overhead by running all the tests in parallel
+    -- lint can make a big different to the command lines, so test with and without
+    whenM hasTracker $
+        build ["-j4","--no-lint"]
     build ["-j4"]
 
 
